@@ -14,7 +14,7 @@ from .johnnycanencrypt import (
     CryptoError,
     Johnny,
     SameKeyError,
-    create_newkey,
+    create_key,
     encrypt_bytes_to_bytes,
     encrypt_bytes_to_file,
     encrypt_file_internal,
@@ -61,6 +61,8 @@ class Key:
         creationtime=None,
         othervalues={},
         oncard: str = "",
+        can_primary_sign: bool = False,
+        primary_on_card: str = "",
     ):
         self.keyvalue = keyvalue
         self.keytype = keytype
@@ -75,6 +77,8 @@ class Key:
         )
         self.othervalues = othervalues
         self.oncard = oncard
+        self.can_primary_sign = can_primary_sign
+        self.primary_on_card = primary_on_card
 
     def __repr__(self):
         return f"<Key fingerprint={self.fingerprint} type={self.keytype.name}>"
@@ -204,9 +208,14 @@ class KeyStore:
             cursor = con.cursor()
             for row in existing_records:
                 oncard = row["oncard"]
+                # The following because this column may not exist at all
+                try:
+                    primary_on_card = row["primary_on_card"]
+                except IndexError:
+                    primary_on_card = ""
                 fingerprint = row["fingerprint"]
-                sql = "UPDATE keys set oncard=? where fingerprint=?"
-                cursor.execute(sql, (oncard, fingerprint))
+                sql = "UPDATE keys set oncard=?, primary_on_card=? where fingerprint=?"
+                cursor.execute(sql, (oncard, primary_on_card, fingerprint))
         # Now let us rename the file
         os.rename(self.dbpath, oldpath)
         self.dbpath = oldpath
@@ -259,6 +268,7 @@ class KeyStore:
         ktype = 1 if keytype else 0
         subkeys = othervalues["subkeys"]
         mainkeyid = othervalues["keyid"]
+        can_primary_sign = othervalues["can_primary_sign"]
         with con:
             cursor = con.cursor()
             # First let us check if a key already exists
@@ -285,8 +295,19 @@ class KeyStore:
                 cursor.execute(sql, (cert, ktype, etime, ctime, key_id))
             else:
                 # Now insert the new key
-                sql = "INSERT INTO keys (keyvalue, fingerprint, keyid, keytype, expiration, creation) VALUES(?, ?, ?, ?, ?, ?)"
-                cursor.execute(sql, (cert, fingerprint, mainkeyid, ktype, etime, ctime))
+                sql = "INSERT INTO keys (keyvalue, fingerprint, keyid, keytype, expiration, creation, can_primary_sign) VALUES(?, ?, ?, ?, ?, ?, ?)"
+                cursor.execute(
+                    sql,
+                    (
+                        cert,
+                        fingerprint,
+                        mainkeyid,
+                        ktype,
+                        etime,
+                        ctime,
+                        can_primary_sign,
+                    ),
+                )
                 # This `key_id` is the database id
                 key_id = cursor.lastrowid
             # Now let us add the subkey and keyid details
@@ -520,8 +541,8 @@ class KeyStore:
         # Regnerate the key object and return it
         return self.get_key(fingerprint)
 
-    def import_cert(self, keypath: str, onplace=False) -> Key:
-        """Imports a given cert from the given path.
+    def import_key(self, keypath: str, onplace=False) -> Key:
+        """Imports a given key from the given file path.
 
         :param path: Path to the pgp key file.
         :param onplace: Default value is False, if True means the keyfile is in the right directory
@@ -630,6 +651,8 @@ class KeyStore:
                 creationtime = result["creation"]
                 keytype = KeyType.SECRET if result["keytype"] else KeyType.PUBLIC
                 oncard = result["oncard"]
+                can_primary_sign = result["can_primary_sign"]
+                primary_on_card = result["primary_on_card"]
 
                 # Now get the uids
                 sql = "SELECT id, value, revoked FROM uidvalues WHERE key_id=?"
@@ -707,6 +730,8 @@ class KeyStore:
                         creationtime,
                         othervalues,
                         oncard,
+                        can_primary_sign,
+                        primary_on_card,
                     )
                 )
         if finalresult:
@@ -787,7 +812,7 @@ class KeyStore:
                         results.append(key)
         return results
 
-    def create_newkey(
+    def create_key(
         self,
         password: str,
         uids: Optional[Union[List[str], str]] = [],
@@ -796,6 +821,7 @@ class KeyStore:
         expiration=None,
         subkeys_expiration=False,
         whichkeys=7,
+        can_primary_sign=False,
     ) -> Key:
         """Returns a public `Key` object after creating a new key in the store
 
@@ -806,6 +832,7 @@ class KeyStore:
         :param expiration: datetime.datetime, default 0 (Never)
         :param subkeys_expiration: Bool (default False), pass True if you want to set the expiry date to the subkeys instead of master key.
         :param whichkeys: Decides which all subkeys to generate, 1 (for encryption), 2 for signing, 4 for authentication. Add the numbers for mixed result.
+        :param can_primary_sign: Boolean to indicate if the primary key can do signing
         """
         if creation:
             ctime = creation.timestamp()
@@ -823,7 +850,7 @@ class KeyStore:
         elif isinstance(uids, list):
             finaluids = uids
 
-        public, secret, fingerprint = create_newkey(
+        public, secret, fingerprint = create_key(
             password,
             finaluids,
             ciphersuite.value,
@@ -831,13 +858,14 @@ class KeyStore:
             int(etime),
             subkeys_expiration,
             whichkeys,
+            can_primary_sign,
         )
         # Now save the secret key
         key_filename = os.path.join(self.path, f"{fingerprint}.sec")
         with open(key_filename, "w") as fobj:
             fobj.write(secret)
 
-        key = self.import_cert(key_filename)
+        key = self.import_key(key_filename)
 
         # TODO: should we remove the key_filename from the disk?
         return key
@@ -1020,7 +1048,7 @@ class KeyStore:
         else:
             return jp.decrypt_filehandler(fh, outputpath, password)
 
-    def sign(self, key, data, password):
+    def sign_detached(self, key, data, password):
         """Signs the given data with the key.
 
         :param key: Fingerprint or secret Key object
@@ -1045,7 +1073,7 @@ class KeyStore:
         jp = Johnny(k.keyvalue)
         return jp.sign_bytes_detached(data, password)
 
-    def verify(self, key, data, signature):
+    def verify(self, key, data, signature: Optional[str]) -> bool:
         """Verifies the given data and the signature
 
         :param key: Fingerprint or public Key object
@@ -1062,15 +1090,63 @@ class KeyStore:
         if type(data) == str:
             data = data.encode("utf-8")
         jp = Johnny(k.keyvalue)
-        return jp.verify_bytes(data, signature.encode("utf-8"))
 
-    def sign_file(self, key, filepath, password, write=False):
+        if signature:
+            return jp.verify_bytes_detached(data, signature.encode("utf-8"))
+        else:
+            return jp.verify_bytes(data)
+
+    def sign_file(self, key, filepath, outputpath, password, cleartext=False) -> bool:
+        """Signs the given input file with key and saves in the outputpath.
+
+        :param key: Fingerprint or secret Key object, public key in case card based operation.
+        :param filepath: str value of the path to the file.
+        :param outputpath: str value of the path to the output signed file.
+        :param password: Password the secret key file or the user pin of the card
+        :param cleartext: If the signed file should be in cleartext or not, default False.
+
+        :returns: Boolean result of the signing operation.
+        """
+        signature = ""
+        if type(key) == str:  # Means we have a fingerprint
+            k = self.get_key(key)
+        else:
+            k = key
+
+        if type(filepath) == str:
+            filepath_in_bytes = filepath.encode("utf-8")
+        else:
+            filepath_in_bytes = filepath
+
+        if type(outputpath) == str:
+            outputpath_in_bytes = outputpath.encode("utf-8")
+        else:
+            outputpath_in_bytes = outputpath
+
+        if k.keytype == KeyType.PUBLIC and k.oncard is not None:
+            result = rjce.sign_file_on_card(
+                k.keyvalue,
+                filepath_in_bytes,
+                outputpath_in_bytes,
+                password.encode("utf-8"),
+                cleartext,
+            )
+
+        else:
+            jp = Johnny(k.keyvalue)
+            result = jp.sign_file(
+                filepath_in_bytes, outputpath_in_bytes, password, cleartext
+            )
+
+        return result
+
+    def sign_file_detached(self, key, filepath, password, write=False):
         """Signs the given data with the key. It also writes filename.asc in the same directory of the file as the signature if write value is True.
 
         :param key: Fingerprint or secret Key object
         :param filepath: str value of the path to the file.
         :param password: Password of the secret key file.
-        :param wrtie: boolean value (default False), determines if we should write the signature to a file.
+        :param write: boolean value (default False), determines if we should write the signature to a file.
 
         :returns: The signature as string
         """
@@ -1102,7 +1178,7 @@ class KeyStore:
 
         return signature
 
-    def verify_file(self, key, filepath, signature_path):
+    def verify_file_detached(self, key, filepath, signature_path):
         """Verifies the given filepath based on the signature file.
 
         :param key: Fingerprint or public Key object
@@ -1130,7 +1206,28 @@ class KeyStore:
         if type(filepath) == str:
             filepath = filepath.encode("utf-8")
         jp = Johnny(k.keyvalue)
-        return jp.verify_file(filepath, signature_in_bytes)
+        return jp.verify_file_detached(filepath, signature_in_bytes)
+
+    def verify_file(self, key, filepath):
+        """Verifies the given filepath.
+
+        :param key: Fingerprint or public Key object
+        :param filepath: File to be verified.
+
+        :returns: Boolean
+        """
+        if type(key) == str:  # Means we have a fingerprint
+            k = self.get_key(key)
+        else:
+            k = key
+
+        if not os.path.exists(filepath):
+            raise FileNotFoundError(f"The file at {filepath} is missing.")
+
+        if type(filepath) == str:
+            filepath = filepath.encode("utf-8")
+        jp = Johnny(k.keyvalue)
+        return jp.verify_file(filepath)
 
     def fetch_key_by_fingerprint(self, fingerprint: str):
         """Fetches key from keys.openpgp.org based on the fingerprint.
@@ -1196,6 +1293,7 @@ class KeyStore:
 
         :returns: The fingerprint of the primary key.
         """
+        fingerprint: str = ""
         data = rjce.get_card_details()
         if not data["serial_number"]:
             return "No data found."
@@ -1217,4 +1315,21 @@ class KeyStore:
                 sql = "SELECT fingerprint from keys where id=?"
                 cursor.execute(sql, (fromdb["key_id"],))
                 result = cursor.fetchone()
-                return result["fingerprint"]
+                fingerprint = result["fingerprint"]
+            # Now let us see if we can find the primary key on the card
+            sql = "SELECT DISTINCT id, fingerprint FROM keys where fingerprint IN (?, ?, ?)"
+            sig_f = convert_fingerprint(data["sig_f"])
+            enc_f = convert_fingerprint(data["enc_f"])
+            auth_f = convert_fingerprint(data["auth_f"])
+            cursor.execute(sql, (sig_f, enc_f, auth_f))
+            fromdb = cursor.fetchone()
+            if fromdb:
+                # Means we found the main key, now we have to mark it with the serial number of the card
+                sql = "UPDATE keys SET primary_on_card=? WHERE id=?"
+                cursor.execute(sql, (data["serial_number"], fromdb["id"]))
+                sql = "SELECT fingerprint from keys where id=?"
+                cursor.execute(sql, (fromdb["id"],))
+                result = cursor.fetchone()
+                fingerprint = result["fingerprint"]
+
+            return fingerprint
