@@ -38,19 +38,26 @@ def test_no_such_key():
         ks = jce.KeyStore("tests/files/store")
         key = ks.get_key("A4F388BBB194925AE301F844C52B42177857DD79")
 
+def test_create_primary_key_with_encryption():
+    ks = jce.KeyStore(tmpdirname.name)
+    newkey = ks.create_key("redhat", "test key42 <42@example.com>", jce.Cipher.RSA4k, whichkeys=1, can_primary_sign=True)
+    assert newkey.can_primary_sign == True
 
 def test_keystore_lifecycle():
+    # Before anything let us first delete if any existing db
+    pathname = os.path.join(tmpdirname.name, "jce.db")
+    if os.path.exists(pathname):
+        os.remove(pathname)
+    # Now create a fresh db
     ks = jce.KeyStore(tmpdirname.name)
-    newkey = ks.create_newkey(
-        "redhat", "test key1 <email@example.com>", jce.Cipher.RSA4k
-    )
+    newkey = ks.create_key("redhat", "test key1 <email@example.com>", jce.Cipher.RSA4k)
     # the default key must be of secret
     assert newkey.keytype == jce.KeyType.SECRET
 
-    ks.import_cert("tests/files/store/public.asc")
-    ks.import_cert("tests/files/store/pgp_keys.asc")
-    ks.import_cert("tests/files/store/hellopublic.asc")
-    ks.import_cert("tests/files/store/secret.asc")
+    ks.import_key("tests/files/store/public.asc")
+    ks.import_key("tests/files/store/pgp_keys.asc")
+    ks.import_key("tests/files/store/hellopublic.asc")
+    ks.import_key("tests/files/store/secret.asc")
     # Now check the numbers of keys in the store
     assert (2, 2) == ks.details()
 
@@ -63,6 +70,9 @@ def test_keystore_lifecycle():
     assert len(keys_via_emails) == 1
     assert key_via_fingerprint == keys_via_emails[0]
 
+    # Also verify that kushal's primary key can sign
+    assert key_via_fingerprint.can_primary_sign == True
+
     # Now verify name cache
     key_via_fingerprint = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
     keys_via_names = ks.get_keys(
@@ -76,7 +86,7 @@ def test_keystore_contains_key():
     "verifies __contains__ method for keystore"
     ks = jce.KeyStore(tmpdirname.name)
     keypath = "tests/files/store/secret.asc"
-    k = ks.import_cert(keypath)
+    k = ks.import_key(keypath)
     _, fingerprint, keytype, exp, ctime, othervalues = jce.parse_cert_file(keypath)
 
     # First only the fingerprint
@@ -116,19 +126,19 @@ def test_keystore_key_uids():
 def test_key_password_change():
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    k = ks.import_cert("tests/files/store/secret.asc")
+    k = ks.import_key("tests/files/store/secret.asc")
     k2 = ks.update_password(k, "redhat", "byebye")
-    data = ks.sign(k2, b"hello", "byebye")
+    data = ks.sign_detached(k2, b"hello", "byebye")
 
 
 def test_key_deletion():
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    ks.import_cert("tests/files/store/public.asc")
-    k = ks.import_cert("tests/files/store/pgp_keys.asc")
-    ks.import_cert("tests/files/store/hellopublic.asc")
-    ks.import_cert("tests/files/store/hellosecret.asc")
-    ks.import_cert("tests/files/store/secret.asc")
+    ks.import_key("tests/files/store/public.asc")
+    k = ks.import_key("tests/files/store/pgp_keys.asc")
+    ks.import_key("tests/files/store/hellopublic.asc")
+    ks.import_key("tests/files/store/hellosecret.asc")
+    ks.import_key("tests/files/store/secret.asc")
     assert (1, 2) == ks.details()
 
     ks.delete_key("F4F388BBB194925AE301F844C52B42177857DD79")
@@ -155,8 +165,8 @@ def test_ks_update_expiry_time_for_subkeys():
     "Updates expiry time for a given subkey"
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    ks.import_cert("tests/files/store/hellosecret.asc")
-    ks.import_cert("tests/files/store/secret.asc")
+    ks.import_key("tests/files/store/hellosecret.asc")
+    ks.import_key("tests/files/store/secret.asc")
 
     key = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
     subkeys = [
@@ -287,7 +297,7 @@ def test_ks_encrypt_decrypt_file_multiple_recipients(encrypt_decrypt_file):
 def test_ks_sign_data():
     ks = jce.KeyStore("tests/files/store")
     key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
-    signed = ks.sign(key, "hello", "redhat")
+    signed = ks.sign_detached(key, "hello", "redhat")
     assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
     assert ks.verify(key, "hello", signed)
 
@@ -295,21 +305,21 @@ def test_ks_sign_data():
 def test_ks_sign_data_fails():
     ks = jce.KeyStore("tests/files/store")
     key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
-    signed = ks.sign(key, "hello", "redhat")
+    signed = ks.sign_detached(key, "hello", "redhat")
     assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
     assert not ks.verify(key, "hello2", signed)
 
 
-def test_ks_sign_verify_file():
+def test_ks_sign_verify_file_detached():
     inputfile = "tests/files/text.txt"
     tempdir = tempfile.TemporaryDirectory()
     shutil.copy(inputfile, tempdir.name)
     ks = jce.KeyStore("tests/files/store")
     key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
     file_to_be_signed = os.path.join(tempdir.name, "text.txt")
-    signed = ks.sign_file(key, file_to_be_signed, "redhat", write=True)
+    signed = ks.sign_file_detached(key, file_to_be_signed, "redhat", write=True)
     assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
-    assert ks.verify_file(key, file_to_be_signed, file_to_be_signed + ".asc")
+    assert ks.verify_file_detached(key, file_to_be_signed, file_to_be_signed + ".asc")
 
 
 def test_ks_creation_expiration_time():
@@ -323,13 +333,13 @@ def test_ks_creation_expiration_time():
     # First let us check from the file
     keypath = "tests/files/store/pgp_keys.asc"
     ks = jce.KeyStore(tmpdir.name)
-    k = ks.import_cert(keypath)
+    k = ks.import_key(keypath)
     assert etime.date() == k.expirationtime.date()
     assert ctime.date() == k.creationtime.date()
 
     # now with a new key and creation time
     ctime = datetime.datetime(2010, 10, 10, 20, 53, 47)
-    newk = ks.create_newkey(
+    newk = ks.create_key(
         "redhat", "Another test key", ciphersuite=jce.Cipher.Cv25519, creation=ctime
     )
     assert ctime.date() == newk.creationtime.date()
@@ -338,16 +348,14 @@ def test_ks_creation_expiration_time():
     # Now both creation and expirationtime
     ctime = datetime.datetime(2008, 10, 10, 20, 53, 47)
     etime = datetime.datetime(2025, 12, 15, 20, 53, 47)
-    newk = ks.create_newkey(
-        "redhat", "Another test key", creation=ctime, expiration=etime
-    )
+    newk = ks.create_key("redhat", "Another test key", creation=ctime, expiration=etime)
     assert ctime.date() == newk.creationtime.date()
     assert etime.date() == newk.expirationtime.date()
 
     # Now both creation and expirationtime for subkeys
     ctime = datetime.datetime(2008, 10, 10, 20, 53, 47)
     etime = datetime.datetime(2029, 12, 15, 20, 53, 47)
-    newk = ks.create_newkey(
+    newk = ks.create_key(
         "redhat",
         "Test key with subkey expiration",
         creation=ctime,
@@ -360,7 +368,7 @@ def test_ks_creation_expiration_time():
 
     # Now only providing expirationtime for subkeys
     etime = datetime.datetime(2030, 6, 5, 20, 53, 47)
-    newk = ks.create_newkey(
+    newk = ks.create_key(
         "redhat",
         "Test key with subkey expiration",
         expiration=etime,
@@ -395,7 +403,7 @@ def test_add_userid():
     """Verifies that we can add uid to a cert"""
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    key = ks.import_cert("tests/files/store/secret.asc")
+    key = ks.import_key("tests/files/store/secret.asc")
     # check that there is only one userid
     assert len(key.uids) == 1
 
@@ -411,7 +419,7 @@ def test_add_and_revoke_userid():
     """Verifies that we can add uid to a cert"""
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    key = ks.import_cert("tests/files/store/secret.asc")
+    key = ks.import_key("tests/files/store/secret.asc")
     # check that there is only one userid
     assert len(key.uids) == 1
 
@@ -442,7 +450,7 @@ def test_add_userid_fails_for_public():
     """Verifies that adding uid to a public key fails"""
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    key = ks.import_cert("tests/files/store/public.asc")
+    key = ks.import_key("tests/files/store/public.asc")
     # verify that the key is a secret
     assert len(key.uids) == 1
 
@@ -472,15 +480,15 @@ def test_update_subkey_expiry_time():
 def test_same_key_import_error():
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    ks.import_cert("tests/files/store/public.asc")
-    with pytest.raises(jce.SameKeyError):
-        ks.import_cert("tests/files/store/public.asc")
+    ks.import_key("tests/files/store/public.asc")
+    with pytest.raises(jce.CryptoError):
+        ks.import_key("tests/files/store/public.asc")
 
 
 def test_key_without_uid():
     tempdir = tempfile.TemporaryDirectory()
     ks = jce.KeyStore(tempdir.name)
-    k = ks.create_newkey("redhat")
+    k = ks.create_key("redhat")
     uids, fp, secret, et, ct, othervalues = jce.parse_cert_bytes(k.keyvalue)
     assert len(uids) == 0
 
@@ -493,7 +501,7 @@ def test_key_with_multiple_uids():
         "kushal@freedom.press",
         "This is also Kushal",
     ]
-    k = ks.create_newkey("redhat", uids)
+    k = ks.create_key("redhat", uids)
     uids, fp, secret, et, ct, othervalues = jce.parse_cert_bytes(k.keyvalue)
     assert len(uids) == 3
 
